/*
 * Project name:
     DEMO
 * Copyright:
     (c) Mikroelektronika, 2011.
 * Description:
      This is a simple demonstration of the GLCD library routines:
     - Init and Clear (pattern fill)
     - Image display
		 - Draw on GLCD using Touch-panel
 * Test configuration:
     MCU:             LPC2214
     Dev.Board:       UNIDS6
                      http://www.mikroe.com/eng/products/view/669/uni-ds6-development-system/
     Oscillator:      14.74568MHz (cclk = 58.98272MHz, Fcco = 235.93088 MHz)
     Ext. Modules:    -
     SW:              KEIL uVision v4.10
 * NOTES:
     - Turn ON the GLCD Backlight at SW18.8
	 - Turn ON SW17.1 - SW17.8 (data lines). Be sure to pull-down data lines!
	 - Turn ON SW18.1 - SW18.6 (ctrl lines). Be sure to pull-up the CS1\ and CS2\ lines!
     - Turn ON four Touch-panel switches SW19.7, SW19.8, SW15.7	and SW15.8.
     - The LEDs on all ports OFF!
*/


#include <LPC22XX.H>

#include "Glcd.h"
#include "Utility.h"



//--- DEFINITIONS
#define Ctrl_IOPIN		IOPIN0
#define Ctrl_IOCLR		IOCLR0
#define Ctrl_IOSET		IOSET0
#define Pin_CS1				2
#define Pin_CS2				3
#define Pin_RS				4
#define Pin_RW				5
#define Pin_EN				6
#define Pin_RST				7

#define Panel_IOPIN		IOPIN0
#define Panel_IODIR		IODIR0
#define Panel_IOCLR		IOCLR0
#define Panel_IOSET		IOSET0
#define Pin_DrvA			24
#define Pin_DrvB			25


//--- CONSTANTS

char 
	write_msg[] = "WRITE",	    // GLCD menu messages
	clear_msg[] = "CLEAR",
	erase_msg[] = "ERASE";

const unsigned long
	ADC_THRESHOLD = 960;		// Threshold value for press detection


//--- VARIABLES
unsigned char
	GLCD_CSstat,
	write_erase,
	pen_size;

unsigned long
	x_coord,    y_coord,
	x_coordOLD, y_coordOLD,
	x_coordD,   y_coordD;

long
	// scaled x-y position
	x_coord128, y_coord64,
	// calibration constants
	cal_x_min, cal_y_min,
	cal_x_max, cal_y_max;



//**************************************************************************************************
//
// ADC Routines
//
//**************************************************************************************************

//---------------------------------------------------------------
// Initialize internal A/D Converter module
//---------------------------------------------------------------
void ADC_Init()
{
	// AD0.1 [P0.28] as TP-BOTTOM analog pin
	// AD0.2 [P0.29] as TP-LEFT analog pin

	// Initial AD0.1, AD0.2 (P0.28, P0.29) By Set PINSEL1[27:26=01, 25:24=01] 
	// xxxx 0101 xxxx xxxx xxxx xxxx xxxx xxxx
	PINSEL1 &= 0xF0FFFFFF;
	PINSEL1 |= 0x05000000;
	// set PDA0 (Enable Power to ADC0)
	PCONP |= (unsigned long)(0x00000001) << 12;
 
	// Enable ADC0
	ADCR &= 0x00000000;		// Clear All Bit Control
	ADCR |= 0x00000004;		// Select ADC	= AD0.2
	ADCR |= 0x00000700;		// ADC Clock	= VBP(PCLK) / 8
	ADCR &= 0xFFFEFFFF;		// Busrt			= 0			- Conversions are software controlled and reguire 11 clocks
	ADCR &= 0xFFF1FFFF;		// CLKS				= 000		- 10Bit : 11 Cycle Clock Conversion
	ADCR |= 0x00200000;		// PDN				= 1			- Active ADC Module
}

//---------------------------------------------------------------
// Read Analog Value from AD0.1 [P0.28] = TP-BOTTOM analog pin
//---------------------------------------------------------------
unsigned long ADC_Get1()  
{
	unsigned long
		val;

	// Select ADC = AD0.1
	ADCR &= 0xFFFFFF00;
	ADCR |= 0x00000002;
	// START = 001 = Start Conversion Now
	ADCR |= 0x01000000;
	// Wait ADC Conversion to Complete and Read A/D Data Register
	do {
		DelayProc(5.0E-06 * CCLOCK);
		val = ADDR;
	} while ((val & 0x80000000) == 0);
	// Shift ADC Result to Integer
	val = (val >> 6) & 0x000003FF;
	ADCR &= 0xF8FFFFFF;

	return (val);
}
//---------------------------------------------------------------
// Read Analog Value from AD0.2 [P0.29] = TP-LEFT analog pin
//---------------------------------------------------------------
unsigned long ADC_Get2()  
{
	unsigned long
		val;

	// Select ADC = AD0.2
	ADCR &= 0xFFFFFF00;
	ADCR |= 0x00000004;
	// START = 001 = Start Conversion Now
	ADCR |= 0x01000000;
	// Wait ADC Conversion to Complete and Read A/D Data Register
	do {
		DelayProc(5.0E-06 * CCLOCK);
		val = ADDR;
	} while ((val & 0x80000000) == 0);
	// Shift ADC Result to Integer
	val = (val >> 6) & 0x000003FF;
	ADCR &= 0xF8FFFFFF;

	return (val);
}
//---------------------------------------------------------------
// Disconnect GLCD module
//---------------------------------------------------------------
void Untouch_GLCD()
{
	// Store state of GLCD pins CS1#, CS2#, RS and R/W.
	GLCD_CSstat = 0;
	if (((Ctrl_IOPIN) & ((unsigned long)(1) << Pin_CS1)) != 0)
		GLCD_CSstat |= 0x01;
	if (((Ctrl_IOPIN) & ((unsigned long)(1) << Pin_CS2)) != 0)
		GLCD_CSstat |= 0x02;
	if (((Ctrl_IOPIN) & ((unsigned long)(1) << Pin_RS)) != 0)
		GLCD_CSstat |= 0x04;
	if (((Ctrl_IOPIN) & ((unsigned long)(1) << Pin_RW)) != 0)
		GLCD_CSstat |= 0x08;
	// Set CS1# and CS2# to disconnect GLCD module
	Ctrl_IOSET |= ((unsigned long)(1) << Pin_CS1);
	Ctrl_IOSET |= ((unsigned long)(1) << Pin_CS2);
}
//---------------------------------------------------------------
// Connect GLCD module
//---------------------------------------------------------------
void Touch_GLCD()
{
	// R/W
	if ( (GLCD_CSstat & 0x08) == 0 )
		Ctrl_IOCLR |= ((unsigned long)(1) << Pin_RW);
	else
		Ctrl_IOSET |= ((unsigned long)(1) << Pin_RW);

	// RS
	if ( (GLCD_CSstat & 0x04) == 0 )
		Ctrl_IOCLR |= ((unsigned long)(1) << Pin_RS);
	else
		Ctrl_IOSET |= ((unsigned long)(1) << Pin_RS);

	// CS2#
	if ( (GLCD_CSstat & 0x02) == 0 )
		Ctrl_IOCLR |= ((unsigned long)(1) << Pin_CS2);
	else
		Ctrl_IOSET |= ((unsigned long)(1) << Pin_CS2);

	// CS1#
	if ( (GLCD_CSstat & 0x01) == 0 )
		Ctrl_IOCLR |= ((unsigned long)(1) << Pin_CS1);
	else
		Ctrl_IOSET |= ((unsigned long)(1) << Pin_CS1);
}
//**************************************************************************************************




//**************************************************************************************************
//
// Touch-Panel Routines
//
//**************************************************************************************************

//------------------------------------------------------------------------------
// returns 1 if TouchPanel is pressed, 0 otherwise
//------------------------------------------------------------------------------
char PressDetect()
{
	unsigned long
		adc_val;

	Untouch_GLCD();

	// PRESS detecting
	Panel_IODIR |= (unsigned long)(1) << Pin_DrvA;
	Panel_IOCLR |= (unsigned long)(1) << Pin_DrvA;			// DRIVEA  (LEFT drive off, RIGHT drive off, TOP drive on)
	Panel_IODIR |= (unsigned long)(1) << Pin_DrvB;
	Panel_IOCLR |= (unsigned long)(1) << Pin_DrvB;			// DRIVEB  (BOTTOM drive off)
	DelayProc(150.0E-06 * CCLOCK);

	adc_val = ADC_Get2();		// Read AD0.2 (READ-Y)

	Touch_GLCD();

	return (adc_val > ADC_THRESHOLD);
}

unsigned long GetX()
{
	unsigned long
		adc_val;

	Untouch_GLCD();
	// reading X
	Panel_IODIR |= (unsigned long)(1) << Pin_DrvA;
	Panel_IOSET |= (unsigned long)(1) << Pin_DrvA;			// DRIVEA (LEFT drive on, RIGHT drive on, TOP drive off)
	Panel_IODIR |= (unsigned long)(1) << Pin_DrvB;
	Panel_IOCLR |= (unsigned long)(1) << Pin_DrvB;			// DRIVEB (BOTTOM drive off)
	DelayProc(150.0E-06 * CCLOCK);

	adc_val = ADC_Get1();		// reading X value from AD0.1 (BOTTOM)

	Touch_GLCD();

	return adc_val;
}

unsigned long GetY()
{
	unsigned long
		adc_val;

	Untouch_GLCD();
	// reading Y
	Panel_IODIR |= (unsigned long)(1) << Pin_DrvA;
	Panel_IOCLR |= (unsigned long)(1) << Pin_DrvA;			// DRIVEA  = 0 (LEFT drive off, RIGHT drive off, TOP drive on)
	Panel_IODIR |= (unsigned long)(1) << Pin_DrvB;
	Panel_IOSET |= (unsigned long)(1) << Pin_DrvB;			// DRIVEB  = 1 (BOTTOM drive on)
	DelayProc(150.0E-06 * CCLOCK);

	adc_val = ADC_Get2();		// reading X value from AD0.2 (LEFT)

	Touch_GLCD();

	return adc_val;
}

void Calibrate()
{
	GLCD_Dot(0,63,1);
	Glcd_Write_Text("TOUCH BOTTOM LEFT",10,3,1);
	while ( !PressDetect() );
	// get calibration constants (reading and compensating TouchPanel nonlinearity)
	cal_x_min = GetX() - 10;
	cal_y_min = GetY() - 10;
	DelayProc(0.1 * CCLOCK);

	Glcd_Fill(0);

	GLCD_Dot(127,0,1);
	Glcd_Write_Text("TOUCH UPPER RIGHT",10,4,1);
	while ( !PressDetect() );
	// get calibration constants (reading and compensating TouchPanel nonlinearity)
	cal_x_max = GetX() + 5;
	cal_y_max = GetY() + 5;
	DelayProc(0.1 * CCLOCK);
}



int main (void)
{
	PINSEL0 = 0;
	PINSEL1 = 0;
	PINSEL2 &= 0x0000000C;
	PINSEL2 |= 0x00000030;
	DelayProc(0.2 * CCLOCK);


	// Initialize Internal ADC
	ADC_Init();

	// Initialize GLCD
	Glcd_Init(&Ctrl_IOPIN,	// Control lines Port
						&IOPIN2,			// Data Lines Port
						Pin_CS1,			// CS1#
						Pin_CS2,			// CS2#
						Pin_RS,				// RS
						Pin_RW,				// R/W#
						Pin_RST,			// RST
						Pin_EN,				// EN
						0);					// D0 data line position

	Glcd_Fill(0);
	Glcd_Write_Text("CALIBRATION", 24, 3, 1);
	DelayProc(0.3 * CCLOCK);

	Glcd_Fill(0);
	Calibrate();
	Glcd_Fill(0);

	Glcd_Write_Text("WRITE ON SCREEN", 24, 4, 1) ;
	DelayProc(0.2 * CCLOCK);
	Glcd_Fill(0);
  
	Glcd_Fill(0);
	Glcd_V_Line(0,7,0,1);
	Glcd_Write_Text(clear_msg,1,0,0);
	Glcd_V_Line(0,7,97,1);
	Glcd_Write_Text(erase_msg,98,0,0);

	// Pen Menu:
	Glcd_Rectangle(41,0,52,9,1);
	Glcd_Box(45,3,48,6,1);
	Glcd_Rectangle(63,0,70,7,1);
	Glcd_Box(66,3,67,4,1);
	Glcd_Rectangle(80,0,86,6,1);
	GLCD_Dot(83,3,1);

	write_erase = 1;
	pen_size = 1;
	x_coordOLD = 0;
	y_coordOLD = 0;

	while(1)
		{
		if (PressDetect())
			{
			// after a PRESS is detected read X-Y and convert it to 128x64 space
			x_coord = GetX() - cal_x_min;
			y_coord = GetY() - cal_y_min;


			if (x_coordOLD >= x_coord)
				x_coordD = x_coordOLD - x_coord;
			else
				x_coordD = x_coord - x_coordOLD;
			if (y_coordOLD >= y_coord)
				y_coordD = y_coordOLD - y_coord;
			else
				y_coordD = y_coord - y_coordOLD;

			x_coordOLD = x_coord;
			y_coordOLD = y_coord;
			if ((x_coordD > 25) || (y_coordD > 25))
				continue;


			x_coord128 = (x_coord * 128l) / (cal_x_max - cal_x_min);
			y_coord64 = (64 -(y_coord *64) / (cal_y_max - cal_y_min));

			if ((x_coord128 < 0) || (x_coord128 > 127))
				continue;
			if ((y_coord64 < 0) || (y_coord64 > 63))
				continue;


			// if clear is pressed
			if ((x_coord128 < 31) && (y_coord64 < 8))
				{
				Glcd_Fill(0);

				// Pen Menu:
				Glcd_Rectangle(41,0,52,9,1);
				Glcd_Box(45,3,48,6,1);
				Glcd_Rectangle(63,0,70,7,1);
				Glcd_Box(66,3,67,4,1);
				Glcd_Rectangle(80,0,86,6,1);
				GLCD_Dot(83,3,1);

				Glcd_V_Line(0,7,0,1);
				Glcd_Write_Text(clear_msg,1,0,0);
				Glcd_V_Line(0,7,97,1);
				if (write_erase)
					Glcd_Write_Text(erase_msg,98,0,0);
				else
					Glcd_Write_Text(write_msg,98,0,0);
				}
         
			// if write/erase is pressed
			if ((x_coord128 > 96) && (y_coord64 < 8))
				{
				if (write_erase)
					{
					write_erase = 0;
					Glcd_Write_Text(write_msg,98,0,0);
					DelayProc(0.1 * CCLOCK);
					}
				else
					{
					write_erase = 1;
					Glcd_Write_Text(erase_msg,98,0,0);
					DelayProc(0.1 * CCLOCK);
					}
				}
         
			// if pen size is selected
			if ((x_coord128 >= 41) && (x_coord128 <= 52) && (y_coord64 <= 9))
				pen_size = 3;

			if ((x_coord128 >= 63) && (x_coord128 <= 70) && (y_coord64 <= 7))
				pen_size = 2;

			if ((x_coord128 >= 80) && (x_coord128 <= 86) && (y_coord64 <= 6))
				pen_size = 1;

			if (y_coord64 < 11)
				continue;

			switch (pen_size)
				{
				case 1:
					GLCD_Dot(x_coord128, y_coord64, write_erase);
					break;
				case 2:
					Glcd_Box(x_coord128, y_coord64, x_coord128 + 1, y_coord64 + 1, write_erase);
					break;
				case 3:
					Glcd_Box(x_coord128-1, y_coord64-1, x_coord128 + 2, y_coord64 + 2, write_erase);
					break;
				}
			}
		}

}

